@@ -1,22 +1,16 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-import six
+from django.core import checks
+from django.core.exceptions import FieldDoesNotExist
+from django.db.models.fields import FloatField, TextField
+from django.utils.encoding import force_text
 
-try:  # noqa
-    from django.core import checks
-except ImportError:  # noqa: Django<1.7
-    pass
-
-from django.db import models
-from django.db.models.fields import TextField, FloatField, FieldDoesNotExist
-from django.utils.encoding import force_text, python_2_unicode_compatible
-
-from .forms import OSMWidget
+from .forms import OSMFormField
 from .validators import validate_latitude, validate_longitude
+from .widgets import OSMWidget
 
 
-@python_2_unicode_compatible
 class Location(object):
     """
     A wrapper class bundling the description of a location (``text``) and its
@@ -55,17 +49,17 @@ class Location(object):
 
     def __eq__(self, other):
         return (
-            isinstance(other, self.__class__) and
-            self.lat == other.lat and
-            self.lon == other.lon and
-            self.text == other.text
+            isinstance(other, self.__class__)
+            and self.lat == other.lat
+            and self.lon == other.lon
+            and self.text == other.text
         )
 
     def __ne__(self, other):
         return not (self == other)
 
 
-class LatitudeField(six.with_metaclass(models.SubfieldBase, FloatField)):
+class LatitudeField(FloatField):
     """
     Bases: :class:`django.db.models.FloatField`
 
@@ -93,7 +87,7 @@ class LatitudeField(six.with_metaclass(models.SubfieldBase, FloatField)):
         return super(LatitudeField, self).formfield(**kwargs)
 
 
-class LongitudeField(six.with_metaclass(models.SubfieldBase, FloatField)):
+class LongitudeField(FloatField):
     """
     Bases: :class:`django.db.models.FloatField`
 
@@ -121,7 +115,7 @@ class LongitudeField(six.with_metaclass(models.SubfieldBase, FloatField)):
         return super(LongitudeField, self).formfield(**kwargs)
 
 
-class OSMField(six.with_metaclass(models.SubfieldBase, TextField)):
+class OSMField(TextField):
     """
     Bases: :class:`django.db.models.TextField`
 
@@ -136,6 +130,7 @@ class OSMField(six.with_metaclass(models.SubfieldBase, TextField)):
     def __init__(self, *args, **kwargs):
         self._lat_field_name = kwargs.pop('lat_field', None)
         self._lon_field_name = kwargs.pop('lon_field', None)
+        self.data_field_name = kwargs.pop('data_field', None)
         super(OSMField, self).__init__(*args, **kwargs)
 
     def contribute_to_class(self, cls, name):
@@ -203,15 +198,24 @@ class OSMField(six.with_metaclass(models.SubfieldBase, TextField)):
 
     def formfield(self, **kwargs):
         """
-        :returns: A :class:`~django.forms.CharField` with a
-            :class:`~osm_field.forms.OSMWidget`.
+        :returns: A :class:`~osm_field.forms.OSMFormField` with a
+            :class:`~osm_field.widgets.OSMWidget`.
         """
-        widget = OSMWidget(lat_field=self.latitude_field_name,
-            lon_field=self.longitude_field_name)
-        kwargs.update({
-            'widget': widget,
-        })
-        return super(OSMField, self).formfield(**kwargs)
+        widget_kwargs = {
+            'lat_field': self.latitude_field_name,
+            'lon_field': self.longitude_field_name,
+        }
+
+        if self.data_field_name:
+            widget_kwargs['data_field'] = self.data_field_name
+
+        defaults = {
+            'form_class': OSMFormField,
+            'widget': OSMWidget(**widget_kwargs),
+        }
+        defaults.update(kwargs)
+
+        return super(OSMField, self).formfield(**defaults)
 
     @property
     def latitude_field_name(self):
@@ -230,16 +234,3 @@ class OSMField(six.with_metaclass(models.SubfieldBase, TextField)):
         if self._lon_field_name is None:
             self._lon_field_name = self.name + '_lon'
         return self._lon_field_name
-
-
-try:
-    from south.modelsinspector import add_introspection_rules
-    add_introspection_rules(
-        [],
-        [
-            "^osm_field\.fields\.LatitudeField",
-            "^osm_field\.fields\.LongitudeField",
-            "^osm_field\.fields\.OSMField",
-        ])
-except ImportError:
-    pass
